import { ErrorMetadataV8, EventMetadataV8, FunctionMetadataV8, ModuleConstantMetadataV8 } from '../../interfaces/metadata';
import { MetadataInterface } from '../types';
import Option from '../../codec/Option';
import Struct from '../../codec/Struct';
import Vec from '../../codec/Vec';
import Text from '../../primitive/Text';
import { StorageMetadata } from './Storage';
/**
 * @name ModuleMetadataV8
 * @description
 * The definition of a module in the system
 */
export declare class ModuleMetadataV8 extends Struct {
    constructor(value?: any);
    /**
     * @description the module calls
     */
    readonly calls: Option<Vec<FunctionMetadataV8>>;
    /**
     * @description the module constants
     */
    readonly constants: Vec<ModuleConstantMetadataV8>;
    /**
     * @description the module errors
     */
    readonly errors: Vec<ErrorMetadataV8>;
    /**
     * @description the module events
     */
    readonly events: Option<Vec<EventMetadataV8>>;
    /**
     * @description the module name
     */
    readonly name: Text;
    /**
     * @description the associated module storage
     */
    readonly storage: Option<StorageMetadata>;
}
/**
 * @name MetadataV8
 * @description
 * The runtime metadata as a decoded structure
 */
export default class MetadataV8 extends Struct implements MetadataInterface<ModuleMetadataV8> {
    constructor(value?: any);
    /**
     * @description The associated modules for this structure
     */
    readonly modules: Vec<ModuleMetadataV8>;
}
