"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _U8aFixed = _interopRequireDefault(require("./U8aFixed"));

// Copyright 2017-2019 @polkadot/types authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
describe('U8aFixed', () => {
  describe('construction', () => {
    it('allows empty values', () => {
      expect(new _U8aFixed.default().toHex()).toEqual('0x0000000000000000000000000000000000000000000000000000000000000000');
    });
    it('allows construction via with', () => {
      expect(new (_U8aFixed.default.with(64))().bitLength()).toEqual(64);
    });
    it('constructs from hex', () => {
      expect(new (_U8aFixed.default.with(32))('0x01020304').toU8a()).toEqual(new Uint8Array([0x01, 0x02, 0x03, 0x04]));
    });
    it('constructs from number[]', () => {
      expect(new (_U8aFixed.default.with(32))([0x02, 0x03]).toU8a()).toEqual(new Uint8Array([0x02, 0x03, 0x00, 0x00]));
    });
  });
  describe('utils', () => {
    let u8a;
    beforeEach(() => {
      u8a = new _U8aFixed.default([1, 2, 3, 4, 5, 6, 7, 8, 9, 10], 32);
    });
    it('limits the length', () => {
      expect(u8a.length).toEqual(4);
    });
    it('exposes the correct bitLength', () => {
      expect(u8a.bitLength()).toEqual(32);
    });
    it('allows wrapping of a pre-existing instance', () => {
      expect(u8a.toU8a()).toEqual(new Uint8Array([1, 2, 3, 4]));
    });
    it('hash a sane toRawType', () => {
      expect(u8a.toRawType()).toEqual('[u8;4]');
    });
  });
});